# HG changeset patch
# User Johnathan Hansler
# Date 1331357810 28800
# Node ID 4223fbab3d8b79f78f6badcb3967d38c6c3e2acb
# Parent  c239be1054e01526a1b0b62da6691061b9dd5587
Modified the piecewise plotting code. Added the ability to change the xmin and xmax values when plotting piecewise functions, as well as the ability to plot piecewise functions containing infinite intervals. Fixed plot legends with piecewise functions.

diff --git a/sage/functions/piecewise.py b/sage/functions/piecewise.py
--- a/sage/functions/piecewise.py
+++ b/sage/functions/piecewise.py
@@ -1011,7 +1011,78 @@
         and then open it in a graphics viewer such as GIMP.
         """
         from sage.plot.all import plot
-        return sum([plot(f, a, b, *args, **kwds) for (a,b),f in self.list()])
+	from sage.rings.infinity import minus_infinity, infinity
+	
+	# By default, plot everything in the function's domain
+	xmin, xmax = self.domain()
+	
+	# If xmin or xmax are -Infinity or +Infinity, we reassign it to a "sensible"
+	# value: If the piecewise function has n parts, and either the first or last
+	# has a (positive or negative) infinite bound, the plotting function designate
+	# 1/nth of the space in the plot to plot that function. Special cases are also
+	# considered, e.g. when the piecewise function consists of only one function
+	# having infinite bounds.
+	if xmin == minus_infinity or xmax == infinity:
+		if xmin == minus_infinity and xmax != infinity:
+			non_inf_min = self.list()[0][0][1]
+			if self.length() == 1:
+				xmin = xmax - 2
+			else:
+				non_inf_len = xmax - non_inf_min
+				xmin = non_inf_min - non_inf_len / (self.length() - 1)
+		elif xmin != minus_infinity and xmax == infinity:
+			non_inf_max = self.list()[-1][0][0]
+			if self.length() == 1:
+				xmax = xmin + 2
+			else:
+				non_inf_len = non_inf_max - xmin
+				xmax = non_inf_max + non_inf_len / (self.length() - 1)
+		else:
+			non_inf_min = self.list()[0][0][1]
+			non_inf_max = self.list()[-1][0][0]
+			if self.length() == 1:
+				xmin, xmax = (-1, 1)
+			elif self.length() == 2:
+				xmin = non_inf_min - 1
+				xmax = non_inf_max + 1
+			else:
+				non_inf_len = non_inf_max - non_inf_min
+				xmin = non_inf_min - non_inf_len / (self.length() - 2)
+				xmax = non_inf_max + non_inf_len / (self.length() - 2)
+	
+	# if the user supplies xmin and/or xmax, override the default values
+	if type(args[0]) == tuple or type(args[0]) == list:
+		_, xmin, xmax = args[0]
+		args = args[1:]
+	elif len(args) >= 2:
+		xmin, xmax = args[:2]
+		args = args[2:]
+	
+	if 'xmin' in kwds:
+		xmin = kwds['xmin']
+		del(kwds['xmin'])
+	if 'xmax' in kwds:
+		xmax = kwds['xmax']
+		del(kwds['xmax'])
+	
+	# if the user supplies a legend label, add this before summing the plots
+	# to avoid duplicate legend entries 
+	legend_label = None
+	if 'legend_label' in kwds:
+		legend_label = kwds['legend_label']
+		del(kwds['legend_label'])
+	
+	# plot the first function with the legend label
+	(a,b),f = self.list()[0]
+	p = plot(f, max(a, xmin), min(b, xmax), legend_label=legend_label, *args, **kwds)
+	
+	# plot the remaining functions without the legend label
+	for func in self.list()[1:]:
+		(a,b),f = func
+		p += plot(f, max(a, xmin), min(b, xmax), *args, **kwds)
+
+	p.set_axes_range(xmin=xmin, xmax=xmax)
+	return p
  
     def fourier_series_cosine_coefficient(self,n,L):
         r"""
